/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_AKAMARU_H__
#define __KIBA_AKAMARU_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <akamaru.h>

typedef struct _KibaAkamaruObject  KibaAkamaruObject;

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaAkamaruObject {
	AkamaruObject   *object;           /* Physical Object */
	AkamaruSpring   *spring;           /* Spring from KibaAkamaruObject to the Dock anchor */
	AkamaruString   *rope_string;      /* String from KibaAkamaruObject to the next one */
	GList           *wobbly_spring;    /* Spring to all other KibaAkamaruObjects */
};

/**
 * Initiall the Akamaru Model.
 *
 * @param screen_geometry The Model Geometry.
 */
void kiba_akamaru_model_init (GdkRectangle screen_geometry);

/**
 * Creates and returns a new KibaAkamaruObject.
 *
 * @param radius The Radius for the new Object.
 * @return The KibaAkamaruObject created.
 */
KibaAkamaruObject* kiba_akamaru_add_object (double radius);

/**
 * Set Model geometry.
 *
 * @param screen_geometry The new Model geometry.
 */
void kiba_akamaru_model_set_size (GdkRectangle screen_geometry);

/**
 * Calculate new Object Positions in the Engine.
 *
 * @param step_size The stepsize for the Model step.
 */
void kiba_akamaru_model_step (void);

/**
 * Free the Physics Model.
 */
void kiba_akamaru_model_free (void);

/**
 * Free a Object.
 *
 * @param akamaru The KibaAkamaruObject.
 */
void kiba_akamaru_object_free (KibaAkamaruObject *akamaru);

/**
 * Set Object Position.
 *
 * @param akamaru The KibaAkamaruObject.
 * @param x The X-Position of the Object.
 * @param y The Y-Position of the Object.
 */
void kiba_akamaru_object_set_position (KibaAkamaruObject *akamaru,
				       double             x,
				       double             y);

/**
 * Set Object Position.
 *
 * @param akamaru The KibaAkamaruObject.
 * @param gravity_normal The Normal to define the direction.
 */
void kiba_akamaru_object_set_gravity_point (KibaAkamaruObject *akamaru,
					    GdkRectangle       gravity_normal);

/**
 * Set Model Anchor.
 *
 * @param x The X-Position of the Anchor.
 * @param y The Y-Position of the Anchor.
 */
void kiba_akamaru_set_anchor_position (int x,
				       int y);

/**
 * Set Object Drag Anchor.
 *
 * @param akamaru The KibaAkamaruObject.
 * @param x The X-Position of the Anchor.
 * @param y The Y-Position of the Anchor.
 */
void kiba_akamaru_set_mouse_anchor (KibaAkamaruObject *akamaru,
				    double             x,
				    double             y);

/**
 * Bounce a Object.
 *
 * @param akamaru KibaAkamaruObject.
 * @param bounce_point The Bounce Position.
 */
void kiba_akamaru_object_bounce (KibaAkamaruObject *akamaru,
				 GdkRectangle       bounce_point);

/**
 * Throw a Object.
 *
 * @param akamaru The KibaAkamaruObject.
 * @param dx The X-Position difference.
 * @param dy The Y-Position difference.
 * @param throw_gravity The Throw gravity.
 */
void kiba_akamaru_object_throw (KibaAkamaruObject *akamaru,
				int                dx,
				int                dy);

/**
 * Check if a Object is in motion.
 *
 * @param akamaru The KibaAkamaruObject.
 * @return @c TRUE if the KibaAkamaruObject is in motion.  Otherwise
 *         @c FALSE is returned.
 */
gboolean kiba_akamaru_object_moving (KibaAkamaruObject *akamaru);

/**
 * Check if Akamaru is enabled.
 *
 * @return @c TRUE if Akamaru is enabled.  Otherwise
 *         @c FALSE is returned.
 */
gboolean kiba_akamaru_is_enabled (void);

/**
 * Reload the Object Spacer.
 */
void kiba_akamaru_reload_spacers (void);

/**
 * Get the Mouse Anchor Position.
 *
 * @param anchor_x X-Position of mouse Anchor.
 * @param anchor_y Y-Position of mouse Anchor.
 */
void kiba_akamaru_get_mouse_anchor_position (int *anchor_x,
					     int *anchor_y);

/**
 * Unset the Mouse Anchor.
 */
void kiba_akamaru_unset_mouse_anchor ();

/**
 * 
 */
void kiba_akamaru_update_mouse_anchor (int          new_x,
				       int          new_y,
				       double       icon_size,
				       GdkRectangle screen_geometry);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
