/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_DOCK_H__
#define __KIBA_DOCK_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <cairo.h>

#define KIBA_TYPE_DOCK          (kiba_dock_get_type ())
#define KIBA_DOCK(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DOCK, KibaDock))
#define KIBA_DOCK_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_DOCK,  KibaDockClass))
#define KIBA_IS_DOCK(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_IS_DOCK_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_DOCK_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DOCK, KibaDockClass))

#define DEFAULT_ICON_SIZE 40

typedef struct _KibaDock        KibaDock;
typedef struct _KibaDockClass   KibaDockClass;
typedef struct _KibaDockOptions KibaDockOptions;

#include "kiba-render.h"

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaDock {
	GObject           parent_instance;
	KibaDockOptions  *options;
	
	KibaSurface      *bar;
	GList            *separators;
	
	GtkAllocation    allocation;       /* dock-allocation (animated) */
	GtkAllocation    dest_allocation;  /* dock-allocation (aim for allocation while animtion) */
};

struct _KibaDockOptions {
	GSList          *separators;
	gboolean         draw;
	gboolean         show_all_borders;
	gchar           *border_rgba;
	gchar           *background;
	gchar           *rgba_1;
	gchar           *rgba_2;
	gchar           *rgba_3;
	gchar           *corner_topleft;
	gchar           *corner_topright;
	gchar           *corner_bottomleft;
	gchar           *corner_bottomright;
	double           edge_offset;
	double           object_space;
	double           object_margin;
	double           separator_size;
	double           cap_size;
	double           margin;
	double           position_stepsize;
	double           border_width;
	double           bg_affection;
	double           bg_step_size;
	double           bg_radius_1;
	double           bg_radius_2;
	double           background_alpha;
	double           edge_radius;
};

struct _KibaDockClass {
	GObjectClass     parent_class;
};

/**
 * Creates and returns a new KibaDock.
 *
 * @return The KibaDock object created.
 */
KibaDock* kiba_dock_new ();

/**
 * Draw the Dock Surface Buffer to a Drawing Context.
 *
 * @param dock The KibaDock object.
 * @param d The Kiba Layout Data.
 * @param window_geometry The Kiba Window Geometry.
 * @param cr The Drawing Context.
 */
void kiba_dock_draw (KibaDock       *dock,
		     KibaLayoutData *d,
		     GdkRectangle    window_geometry,
		     cairo_t        *cr);

/**
 * Draw the Dock Surface to a Drawing Context.
 *
 * @param dock The KibaDock object.
 * @param d The Kiba Layout Data.
 * @param cr The Drawing Context for the Surface.
 */
void kiba_dock_draw_surface (KibaDock       *dock,
			     KibaLayoutData *d,
			     cairo_t        *cr);

/**
 * Set the _NET_WM_STRUT proberty to the dock allocation.
 *
 * @param dock The KibaDock object.
 * @param d The Kiba Layout Data.
 * @param window The Kiba Window.
 * @param use_strut @c TRUE when strut proberty should be set, otherwise
 *                  @c FALSE if the strut proberty should be unset.
 */
void kiba_dock_set_strut (KibaDock       *dock,
			  KibaLayoutData *d,
			  GdkWindow      *window,
			  gboolean        use_strut);


/**
 * Allocate new Dock Size.
 *
 * @param dock The KibaDock object.
 * @param d The Kiba Layout Data.
 * @param screen_geometry The Screen Geometry.
 * @param object_space The Space, wich is needed by the Objects.
 * @param num_objects Number of Objects on the Dock.
 */
void kiba_dock_size_allocate (KibaDock       *dock,
			      KibaLayoutData *d,
			      GdkRectangle    screen_geometry,
			      double          object_space,
			      double          num_objects,
			      gboolean        zooming);

/**
 * Calculate Dock animate Geometry.
 *
 * @param dock The KibaDock object.
 * @param zooming @c TRUE if auto_hide is enabled, otherwise.
 *                @c FALSE.
 * @return @c TRUE if , otherwise
 *         @c FALSe.
 */
gboolean kiba_dock_animate_position_changes (KibaDock  *dock,
					     gboolean   zooming);

/**
 * Calculate Dock animate Geometry.
 *
 * @param dock The KibaDock object.
 * @param d The Kiba Layout Data.
 * @param screen_geometry The Screen Geometry.
 * @param stepsize The Animation Stepsize.
 * @param hide @c TRUE if auto_hide is enabled, otherwise.
 *             @c FALSE.
 */
void kiba_dock_calculate_geometry (KibaDock       *dock,
				   KibaLayoutData *d,
				   GdkRectangle    screen_geometry,
				   int             stepsize,
				   gboolean        hide);

/**
 * Free allocated memory of KibaDock.
 *
 * @param dock The KibaDock object.
 */
void kiba_dock_fini (KibaDock *dock);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
