/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_OBJECT_H__
#define __KIBA_OBJECT_H__

#include "../config.h"

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>
#include <pango/pango.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#ifdef HAVE_SVG
  #include <librsvg/rsvg.h>
  #include <librsvg/rsvg-cairo.h>
#endif

typedef struct _KibaObject         KibaObject;
typedef struct _KibaObjectClass    KibaObjectClass;

#include "kiba.h"
#include "kiba-akamaru.h"
#include "kiba-render.h"
#include "kiba-progress.h"
#include "kiba-bubble.h"

#define KIBA_TYPE_OBJECT          (kiba_object_get_type ())
#define KIBA_OBJECT(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_OBJECT, KibaObject))
#define KIBA_OBJECT_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_OBJECT,  KibaObjectClass))
#define KIBA_IS_OBJECT(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_IS_OBJECT_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_OBJECT_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_OBJECT, KibaObjectClass))

#define FINGER_STEP 0.01

#ifdef __cplusplus
extern "C" {
#endif

enum {
	KIBA_OBJECT_PRELIGHT = 0,
	KIBA_OBJECT_TITLE,
	KIBA_OBJECT_PULSE,
	KIBA_OBJECT_URGENT,
	KIBA_OBJECT_RECTANGLE,
	KIBA_OBJECT_DRAGGING,
};

/* this struct savves a kiba objects, like a launcher, clock... */
struct _KibaObject {
	GObject            parent;
	int                flags;	/* object flags */
	
	KibaAkamaruObject  *akamaru;
	KibaProgress       *progress;
	KibaBubble         *bubble;
	KibaSurface        *bgsurface;
	KibaSurface        *fgsurface;
	KibaSurface        *title_surface;
	
	gboolean                  at_dock;
	gboolean                  exec;       /* true on exec */
	char                     *name;       /* Titel for pango rendering */
	char                     *exec_command;
	int                       exec_pid;
	int                       position;   /* position of object in plugin (1=leftmost for horizontal docks) */
	GdkRectangle              geometry;   /* saves width/height x/y of the object, when the object is in motion,
						this is the aim geometry */
	GdkRectangle              animate;    /* saves width/height x/y of the object, when the object is in motion,
						this is the dynamic geometry */
	
	GdkPixbuf                *bg_pixbuf;
	GdkPixbuf                *fg_pixbuf;
#ifdef HAVE_SVG
	RsvgHandle               *bg_svg_handle;
	RsvgHandle               *fg_svg_handle;
#endif
	
	char                     *title;
	PangoLayout              *pango_layout;       //saves the pango layout
	
	double                    alpha;
	double                    pulse_size;  /* scaled pulse size */
	double                    drawn_rotate_angle;
	double                    rotate_angle;
	double                    scale_factor; /* scale factor for rotation */
	double                    normal_size;  /* unscalled size */
	double                    size;         /* scalled size */
	double                    zoom_size;    /* used to avoid to big stepsizes */
};

struct _KibaObjectClass {
	GObjectClass	parent_class;
	
	void (* button_released) (KibaObject *object,
				  gint        button);
	void (* scroll_event) (KibaObject *object,
			       gint        direction);
	void (* drag_event) (KibaObject       *object,
			     GdkDragContext   *context,
			     GtkSelectionData *data);
};

/**
 * Get the Object List.
 *
 * @return The GList of all initialled objects.
 */
GList* kiba_objects_get ();

/**
 * 
 */
void kiba_object_add_switch_surfaces (KibaObject  *object,
				      gboolean     switch_bg,
				      gboolean     switch_fg,
				      const gchar *old_title,
				      gint         old_flags);

/**
 * Creates and returns a new KibaObject.
 *
 * @param settings The KibaSettings struct.
 * @param object_name The Name of the object.
 * @param width The Object Width.
 * @param height The Object Height.
 * @return The KibaObject object created.
 */
KibaObject* kiba_object_new (const char  *object_name,
                             double       width,
                             double       height);

/**
 * Get the Object under the Mouse Pointer.
 *
 * @param window The Kiba Window.
 * @param window_geometry The Kiba Window geometry.
 * @param screen_geometry The Screen Geometry.
 * @return The found KibaObject or NULL.
 */
KibaObject* kiba_object_get_under_pointer (GdkWindow    *window,
					   GdkRectangle  window_geometry,
					   GdkRectangle  screen_geometry);

/**
 * Get the Object with the biggest size.
 *
 * @return The found KibaObject or NULL.
 */
KibaObject* kiba_object_get_biggest (void);

/**
 * Free/Destroy a Object.
 *
 * @param object The KibaObject to free.
 */
void kiba_object_free (KibaObject *object);

/**
 * Launch a Object Command (timeout).
 *
 * @param data The KibaObject to exec.
 * @return @c TRUE as long as it is permitted by Flags to execute the Command, otherwise
 *         @c FALSE when no Command found, or the Command was launched.
 */
gint kiba_object_exec (gpointer data);

/**
 * Hide all initialled Objects.
 *
 * @param kiba The Kiba struct.
 */
void kiba_objects_hide (Kiba *kiba);

/**
 * Update the Geometry for a object.
 *
 * @param object The KibaObject.
 * @param stepsize The maximal Stepsize for The geometry step.
 */
void kiba_object_calculate_geometry (Kiba       *kiba,
				     KibaObject *object,
				     int         stepsize);

/**
 * Bounce a Object.
 *
 * @param object The KibaObject to bounce.
 * @param d The KibaLayoutData.
 * @param bounce_height The Height for the Bounce.
 * @param physics
 */
void kiba_object_bounce (KibaObject     *object,
			 KibaLayoutData *d,
			 double          bounce_height,
			 gboolean        physics);

/**
 * Render a Object Surface Buffer.
 *
 * @param object The KibaObject to update the Surface.
 * @param drawing_context The Cairo context to render on.
 * @param background @c TRUE if the background Surface Buffer should be drawn, otherwise
 *                   @c FALSE.
 */
void kiba_object_draw_surface_buffer (KibaObject *object,
                                      cairo_t    *drawing_context,
                                      gboolean   background);

/**
 * 
 */
void kiba_object_draw (Kiba       *kiba,
		       KibaObject *object,
		       cairo_t    *cr);

/**
 * Load an icon for a KibaObject.
 *
 * @param object The KibaObject to update the Surface.
 * @param icon The Icon name or path.
 * @return @c TRUE if a icon was loaded, otherwise
 *         @c FALSE when no Icon was found.
 */
gboolean kiba_object_load_icon (KibaObject *object,
				const char *icon);

/**
 * Set the _NET_WM_ICON_GEOMETRY proberty on a Window for a Object.
 *
 * @param object The KibaObject to set the proberty to.
 * @param d The KibaLayout struct.
 * @param window The GdkWindow to set the proberty on.
 * @param screen_geometry The Screen Geometry.
 */
void kiba_object_set_icon_geometry (KibaObject     *object,
				    KibaLayoutData *d,
				    GdkWindow      *window,
				    GdkRectangle    screen_geometry);

/**
**/
void kiba_object_zoom_step (Kiba         *kiba,
			    KibaObject   *object,
                            double step_size);

/**
 * Set a KibaObject Position.
 *
 * @param kiba The Kiba Widget.
 * @param object The KibaObject.
 */
void kiba_object_set_position (Kiba       *kiba,
			       KibaObject *object);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
