/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_PLUGIN_H__
#define __KIBA_PLUGIN_H__

#include <gtk/gtk.h>
#include <glib-object.h>
#include <gdk/gdk.h>
#include <glib.h>

typedef struct _KibaPlugin       KibaPlugin;
typedef struct _KibaPluginClass  KibaPluginClass;
typedef struct _KibaPluginVtable KibaPluginVtable;

#include "kiba.h"
#include "kiba-object.h"
#include "kiba-prefs.h"

#define KIBA_TYPE_PLUGIN          (kiba_plugin_get_type ())
#define KIBA_PLUGIN(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_PLUGIN, KibaPlugin))
#define KIBA_PLUGIN_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_PLUGIN,  KibaPluginClass))
#define KIBA_IS_PLUGIN(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_PLUGIN))
#define KIBA_IS_PLUGIN_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_PLUGIN))
#define KIBA_PLUGIN_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_PLUGIN, KibaPluginClass))

/**
 * Handles the initialization of modules.
 */
#define KIBA_INIT_PLUGIN(PrefType, plugin_settings, vtable) \
	gboolean kiba_init_plugin(KibaPlugin *plugin);\
	gboolean kiba_init_plugin(KibaPlugin *plugin) { \
		PrefType *options = g_new0 (PrefType, 1); \
		plugin->option_struct	= options; \
		plugin->settings	= plugin_settings; \
		plugin->option_elements	= G_N_ELEMENTS (plugin_settings); \
		plugin->vtable		= &(vtable); \
		return kiba_plugin_register (plugin); \
	}

#ifdef __cplusplus
extern "C" {
#endif

/**
 * A Plugin VTable.
 */
struct _KibaPluginVtable {
	const char *name;		/* plugin id/name        */
	const char *summary;		/* plugin summary        */
	const char *description;	/* plugin description    */
	
	const char *author;		/* plugin author         */
	const char *homepage;		/* plugin homepage       */
	
	void* (*load) (GtkWidget  *dock,
		       KibaPlugin *plugin);
	void  (*unload) (KibaPlugin *plugin);
	void  (*paint) (KibaPlugin  *plugin,
			KibaObject  *object,
			cairo_t     *cr,
			GdkRectangle offset);
	void  (*surface_buffer) (KibaPlugin *plugin,
				 KibaObject *object,
				 cairo_t    *cr,
				 gboolean    background);
	void  (*context_menu) (KibaPlugin *plugin,
			       KibaObject *object,
			       GtkMenu    *menu,
			      GdkEvent    *event);
};

/**
 * A Plugin Handle.
 */
struct _KibaPlugin {
	GObject            parent_instance;
	char               *name;            /* library filename without extension */
	char               *lib_path;
	
	gboolean           enabled;          /* represents $(plugin_name)_enable from Prefs */
	int                position;         /* plugin position in the dock */
	
	GList              *objects;         /* List of all KibaObjects managed by this Plugin */
	double             size;
	
	KibaPluginVtable   *vtable;
	void               *plugin_struct;   /* plugin struct */
	
	PrefMember         *settings;        /* plugin settings */
	void               *option_struct;   /* plugin option struct */
	int                option_elements;
};

struct _KibaPluginClass {
	GtkWidgetClass parent_class;
	
	void (* object_added)   (KibaPlugin *plugin, KibaObject *object);
	void (* object_removed) (KibaPlugin *plugin, KibaObject *object);
	void (* size_changed)   (KibaPlugin *plugin);
	void (* unloaded)       (KibaPlugin *plugin);
};

/**
 * Creates and returns a new KibaPlugin widget.
 *
 * @param library_path The path to the Plugin Library.
 * @return The KibaPlugin widget created.
 */
KibaPlugin* kiba_plugin_new (char *library_path);

/**
 * Load KibaPlugin.
 *
 * @param dock The Kiba Widget.
 * @param plugin The KibaPlugin.
 */
void kiba_plugin_load (GtkWidget  *dock,
                       KibaPlugin *plugin);

/**
 * UnLoad KibaPlugin.
 *
 * @param plugin The KibaPlugin.
 */
void kiba_plugin_unload (KibaPlugin *plugin);

/**
 * Initiall a KibaObject and add it to KibaPlugin.
 *
 * @param plugin The KibaPlugin.
 * @param object_name The KibaObject name.
 * @param width The KibaObject width.
 * @param height The KibaObject height.
 * @return The KibaObject created.
 */
KibaObject* kiba_plugin_add_object (KibaPlugin *plugin,
				    const char *object_name,
				    double      width,
				    double      height);

/**
 * Destroy a KibaObject and remove it from KibaPlugin.
 *
 * @param plugin The KibaPlugin.
 * @param object The KibaObject.
 */
void kiba_plugin_remove_object (KibaPlugin *plugin,
				KibaObject *object);

/**
 * Launch Function for all KibaObject's.
 *
 * @param for_all_func The Function to launch for every KibaObject.
 * @param data The UserData for the Function.
 */
void kiba_plugin_for_all_objects (GFunc  for_all_func,
				  void  *data);

/**
 * Launch Function for all loaded KibaPlugin's.
 *
 * @param for_all_func The Function to launch for every KibaPlugin.
 * @param data The UserData for the Function.
 */
void kiba_plugin_for_all (GFunc  for_all_func,
			  void  *data);

/**
 * Get loaded KibaPlugin's.
 *
 * @return The loaded KibaPlugin's.
 */
GList* kiba_plugin_get_loaded (void);

/**
 * Get a Pointer to a loaded KibaPlugin.
 *
 * @param name The KibaObject of the Plugin to search for.
 * @return The found KibaPlugin or NULL if no plugin with that KibaObject was loaded.
 */
KibaPlugin* kiba_plugin_get_for_object (KibaObject *object);

/**
 * Get a Pointer to a loaded KibaPlugin.
 *
 * @param name The Name of the Plugin to search for.
 * @return The found KibaPlugin or NULL if no plugin with that Name was loaded.
 */
KibaPlugin* kiba_plugin_get_for_name (const char *name);

/**
 * Get a Pointer to a loaded KibaPlugin.
 *
 * @param name The ID of the Plugin to search for.
 * @return The found KibaPlugin or NULL if no plugin with that ID was loaded.
 */
KibaPlugin* kiba_plugin_get_for_id (const char *id);

/**
 * Get a Pointer to a loaded KibaPlugin.
 *
 * @param setting_name The Pref Key.
 * @return The found KibaPlugin or NULL.
 */
KibaPlugin* kiba_plugin_get_for_option (const char *setting_name);

/**
 * Get a Pointer to a Plugin Struct.
 *
 * @param name The Name of the Plugin to search for.
 * @return The found Plugin Struct or NULL is no plugin with that Name was loaded.
 */
void* kiba_plugin_get_struct_for_name (const char *plugin_name);

/**
 * Unload/Disable a KibaPlugin.
 *
 * @param widget The Menu Item GtkWidget.
 * @param data The KibaPlugin to disable.
 */
void kiba_plugin_disable (GtkWidget *widget,
			  gpointer data);

/**
 * Set size for all Objects managed by Plugin.
 *
 * @param plugin The KibaPlugin.
 * @param size The new size for Objects.
 */
void kiba_plugin_set_size (KibaPlugin *plugin,
			   double      size);

/**
 * Find closest KibaPlugin Position for the Plugin of a KibaObject.
 *
 * @param dragging_object The KibaObject.
 * @param mouse_x The Mouse X-Position.
 * @param mouse_y The Mouse Y-Position.
 */
void kiba_plugin_find_closest_position (KibaObject *dragging_object,
					int         mouse_x,
					int         mouse_y);

/**
 * Set Positions for all KibaObject's.
 *
 * @param geometry The KibaGeometry object.
 */
void kiba_plugin_set_object_positions (Kiba *kiba);

/**
 *
 * @param dock The KibaDock GtkWidget.
 * @param setting_name The Pref Name.
 * @param setting_type The Pref Type.
 * @param setting_value The Pref Value.
 * @return @c TRUE if the Pref found and set successfully.  Otherwise
 *         @c FALSE is returned.
 */
gboolean kiba_plugin_option_changed (GtkWidget  *dock,
				     const char *setting_name,
				     const char *setting_type,
				     const char *setting_value);

/**
**/
void kiba_plugin_sort_plugins (void);

/**
 * Free and Destroy All initialled KibaPlugin's.
 *
 */
void kiba_plugins_free (void);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
