/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_PREFS_H__
#define __KIBA_PREFS_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <glib-object.h>

/********
Preferences
*********/

typedef struct _PrefMember     PrefMember;
typedef struct _KibaPrefOption KibaPrefOption;
typedef struct _KibaPrefs      KibaPrefs;

#include "gfilemonitor.h"

#ifdef __cplusplus
extern "C" {
#endif

/* value changed callback */
typedef void (*KibaPrefCallback) (GtkWidget*, GObject*, const char*);

struct _PrefMember {
	const char           *name;		/* setting option	*/
	const char           *default_value;	/* default value	*/
	GType                type;		/* value type		*/
	KibaPrefCallback     changed_callback;	/* value changed callback */
	size_t               offset;		/* offset in the associated pref struct */
};

struct _KibaPrefs {
	GFileMonitor   *settings_monitor;
};

/**
 * Creates and returns a new KibaPrefs object.
 *
 * @return The KibaPrefs object created.
 */
KibaPrefs* kiba_prefs_get (void);

/**
 * Get Double Value from Kiba Preferences Database.
 *
 * @param option_name The Option Name to watch for.
 * @return The Value, or 0.0 if none Setting was found.
 */
double kiba_prefs_get_double (const char *option_name);

/**
 * Get Bool Value from Kiba Preferences Database.
 *
 * @param option_name The Option Name to watch for.
 * @return The Value, or FALSE if none Setting was found.
 */
gboolean kiba_prefs_get_bool (const char *option_name);

/**
 * Get Int Value from Kiba Preferences Database.
 *
 * @param option_name The Option Name to watch for.
 * @return The Value, or -1 if none Setting was found.
 */
gint kiba_prefs_get_int (const char *option_name);

/**
 * Get String Value from Kiba Preferences Database.
 *
 * @param option_name The Option Name to watch for.
 * @return The Value, or NULL if none Setting was found.
 */
gchar* kiba_prefs_get_string (const char *option_name);

/**
 * Get GList Value from Kiba Preferences Database.
 *
 * @param option_name The Option Name to watch for.
 * @return GList, or NULL if none Setting was found.
 */
GSList* kiba_prefs_get_list (const char *option_name);

/**
 * Get String Value from Kiba Config File.
 *
 * @param option The Option Name to watch for.
 * @param type The Type of the Option.
 * @return The Value as String, or NULL if none Setting was found.
 */
char* kiba_prefs_get_option (const char *option,
                             const char *type);

/**
 * Write Option to Config File.
 *
 * @param option The Pref Name.
 * @param type The Pref Type.
 * @param value The Pref Value.
 * @return TRUE on success.
 */
gboolean kiba_prefs_write_option (const char *option,
                                  const char *type,
                                  const char *value);

/**
 * Append Item to list and write List-Option to Config File.
 *
 * @param list_name The GList Name.
 * @param new_item The New Item to append.
 */
void kiba_prefs_append_to_list (const char *list_name,
                                const char *new_item);

/**
 * Remove Item from list and write List-Option to Config File.
 *
 * @param list_name The GList Name.
 * @param new_item The New Item to append.
 */
void kiba_prefs_remove_from_list (const char *list_name,
                                  const char *remove_item);

/**
 * Set New Value for Preferences Member and call the changed_callback.
 *
 * @param kiba The Kiba Widget.
 * @param option_name The Pref Name.
 * @param option_type The Pref Type.
 * @param option_value The Pref Value.
 */
void kiba_prefs_set_option (GtkWidget  *kiba,
                            const char *option_name,
                            const char *option_type,
                            const char *option_value);

/**
 * Remove previously added Options from the Preferences Database.
 *
 * @param options An array of PrefMember's to remove.
 */
void kiba_prefs_remove_options (PrefMember *options);

/**
 * Initiall Options in the Preferences Database.
 *
 * @param option The Option Struct wich contains the Prefs Member.
 * @param options An array of PrefMember's.
 * @param option_elements Element number of Option Struct.
 */
void kiba_prefs_add_options (void       *option,
                             PrefMember *options,
                             gint        option_elements);

/**
 * Sets Pref to Option Struct.
 *
 * @param option_struct The Option Struct wich contains the Prefs Member.
 * @param options An array of PrefMember's.
 * @param array_elemets Element number of Option Struct.
 * @param setting_name The Pref Name.
 * @param setting_type The Pref Type.
 * @param setting_value The Pref Value.
 */
void kiba_prefs_set_struct_member (void              *option_struct,
                                   const PrefMember  options[],
                                   int               array_elemets,
                                   const char        *setting_name,
                                   const char        *setting_type,
                                   const char        *setting_value);

/**
 * Get a PrefMember for a Pref Name.
 *
 * @param options  An array of PrefMember's.
 * @param array_elemetsElement number of Option Struct.
 * @param setting_name The Pref Name.
 * @return The PrefMember associated with the Pref Name.
 */
const PrefMember* kiba_prefs_get_member_for_name (const PrefMember options[],
                                                  int              array_elemets,
                                                  const char       *setting_name);

/**
 * Returns the Prefs File as allocated string.
 *
 * @return allocated config file string.
 */
char* kiba_prefs_get_config_file (void);

/**
 * Write Prefs from Schema file to Config file.
 *
 * @param kiba_home_dir Directory wich contains the Prefs File.
 * @return Path to the Prefs File.
 */
char* kiba_prefs_update_config_file (const char *kiba_home_dir);

/**
 * Frees allocated Memory.
 */
void kiba_prefs_fini (void);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

/******
Schema
*******/

#define MAX_SECTIONS    50
#define MAX_SUBSECTIONS 30
#define MAX_MEMBER      1000

#define SCHEMA_FILE_SUFFIX       ".xml"

#define XML_SECTION              "section"
#define XML_SECTION_NAME         "sec_name"
#define XML_SECTION_DESC         "sec_desc"
#define XML_SECTION_ICON         "sec_icon"
#define XML_SUBSECTION           "subsection"
#define XML_SUBSECTION_NAME      "sub_sec_name"
#define XML_OPTION               "option"
#define XML_NAME                 "name"
#define XML_TITLE                "title"
#define XML_ICON                 "icon"
#define XML_DEFAULT              "default"
#define XML_TYPE                 "type"
#define XML_RANGE                "range"
#define XML_DESCRIPTION          "description"
#define DEFAULT_SUBSECTION_NAME  "General"

typedef struct _KibaSchema        KibaSchema;
typedef struct _SchemaSection     SchemaSection;
typedef struct _SchemaSubSection  SchemaSubSection;
typedef struct _SchemaMember      SchemaMember;

#ifdef __cplusplus
extern "C" {
#endif

struct _SchemaMember {
	char  *section_name;
	char  *subsection_name;
	char  *widget_name;
	char  *title;
	char  *type;
	char  *range;
	char  *value;
	char  *default_value;
	char  *description;
};

struct _SchemaSubSection {
	gchar         *subsection_name;
	gchar         *section_name;     /* do not free */
	SchemaMember  *options[MAX_MEMBER];
	gint           num_member;
};

struct _SchemaSection {
	gchar             *section_name;
	gchar             *description;
	gchar             *icon;
	SchemaSubSection  *subsections[MAX_SUBSECTIONS];
	gint               num_subsections;
};

struct _KibaSchema {
	SchemaSection  *sections[MAX_SECTIONS];
	gint            num_sections;
};

/**
 * Creates and returns a new KibaSchema.
 *
 * @return The created KibaSchema.
 */
KibaSchema* kiba_prefs_get_schema (void);

/****************
KeyFile file handling
****************/

/**
 * Get a Desktop File Value for the Key Name.
 *
 * @param desktop_file The Path to the Desktop File.
 * @param fallback_exec The Fallback Exec.
 * @param workspace The Workspace to launch the App on.
 * @param screen The Screen to launch the desktop file on.
 * @param error The returned Error.
 */
void kiba_prefs_launch_desktop_file (const char *desktop_file,
                                     const char *fallback_exec,
                                     int         workspace,
                                     GdkScreen  *screen,
                                     GError    **error);

/**
 * Get a Desktop File Value for the Key Name.
 *
 * @param filename The Path to the Desktop File.
 * @param key The Key to look for.
 * @return The Desktop File Value.
 */
char* kiba_prefs_get_desktop_entry (const char *filename,
                                    const char *key);

/**
 * Get a Desktop File Value from a GKeyFile.
 *
 * @param file The GKeyFile.
 * @param key The Key to look for.
 * @return The Desktop File Value.
 */
char* kiba_prefs_get_desktop_entry_from_keyfile (GKeyFile   *file,
                                                 const char *key);

/**
 * Write Desktop File Value for the Key Name in Desktop File.
 *
 * @param desktop_file The Path to the Desktop File.
 * @param desktop_entry The Key to look for.
 * @param new_value The New Value for the Key.
 * @return TRUE on success.
 */
gboolean kiba_prefs_write_desktop_entry (const char  *desktop_file,
                                         const char  *desktop_entry,
                                         const char  *new_value);

/**
 * Copy KeyFile at File Path to Kiba Home Directory.
 *
 * @param desktop_path TFile Path to the Key File.
 */
void kiba_prefs_merge_desktop_to_home (const char *desktop_path);

/**
 * Add all missing Keys from the Key Array to the Key File.
 *
 * @param uri The Path to the Key File.
 * @param key_entry_group The Entry Group to look for.
 * @param fields The Key Array of wanted Keys in the File.
 * @param elements Element number of the Key Array.
 * @return TRUE on success.
 */
gboolean kiba_prefs_add_missing_desktop_entries (const char  *uri,
                                                 const char  *key_entry_group,
                                                 const char  **fields,
                                                 int         elements);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
