/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_PROGRESS_H__
#define __KIBA_PROGRESS_H__

#include <glib.h>
#include <cairo.h>

typedef struct _KibaProgress         KibaProgress;
typedef struct _KibaProgressClass    KibaProgressClass;

#include "kiba-render.h"

#define KIBA_TYPE_PROGRESS          (kiba_progress_get_type ())
#define KIBA_PROGRESS(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_PROGRESS, KibaProgress))
#define KIBA_PROGRESS_CLASS(obj)    (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_PROGRESS,  KibaProgressClass))
#define KIBA_IS_PROGRESS(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_PROGRESS))
#define KIBA_IS_PROGRESS_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_PROGRESS))
#define KIBA_PROGRESS_GET_CLASS     (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_PROGRESS, KibaProgressClass))

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaProgress {
	GObject    parent;
	
	char      *message;       /* message for the bubble */
	double     value;         /* current value of the process */
	double     dest_value;    /* dest value of progress */
	double     fraction;      /* drawn fraction */
};

struct _KibaProgressClass {
	GObjectClass parent_class;
};

/**
 * Create a KibaProgress Instance.
 *
 * @return The KibaProgress Instance created.
 */
KibaProgress* kiba_progress_new (void);

/**
 * Start new progress for a KibaProgress Instance.
 *
 * @param progress The KibaProgress Instance.
 * @param dest_value The destination value of the Progress.
 */
void kiba_progress_start (KibaProgress *progress,
			  double        dest_value);

/**
 * Stop progress for a KibaProgress Instance.
 *
 * @param progress The KibaProgress Instance.
 */
void kiba_progress_stop (KibaProgress *progress);

/**
 * Set progress info for a KibaProgress Instance.
 *
 * @param progress The KibaProgress Instance.
 * @param value The new progress value (must be <= dest_value).
 * @param message The new Progress message or NULL.
 */
void kiba_progress_set_info (KibaProgress *progress,
			     double        value,
			     const char   *message);

/**
 * Return if a progress for a KibaProgress Instance is running.
 *
 * @param progress The KibaProgress Instance.
 * @return @c TRUE  if a progress, started with kiba_progress_start, is running, otherwise
 *	   @c FALSE is returned.
 */
gboolean kiba_progress_is_running (KibaProgress *progress);

/**
 * Draw the progress Surface to a Drawing context.
 *
 * @param cr The Drawing Context.
 * @param radius The Object Radius.
 * @param rgba The Progress RGBA.
 * @param bg_rgba The Progress Background RGBA.
 */
void kiba_progress_draw_surface (cairo_t    *cr,
				 double      radius,
				 const char *rgba,
				 const char *bg_rgba);

/**
 * Draw the progress Surface-Buffer to a Drawing context.
 *
 * @param progress The KibaProgress Instance.
 * @param surface The KibaSurface.
 * @param cr The Drawing Context.
 * @param width The Object Width.
 * @param width The Object Height.
 * @param rgba The Progress RGBA.
 * @return @c 1 if a progress is over,
 *	   @c 0 if a progress is active, otherwise
 *	   @c -1 is returned.
 */
int kiba_progress_draw (KibaProgress *progress,
			KibaSurface  *surface,
			cairo_t      *cr,
			double        width,
			double        height,
			const char   *rgba);

/**
 * Free allocated memory for a KibaProgress Instance.
 *
 * @param progress The KibaSettings Instance.
 */
void kiba_progress_fini (KibaProgress *progress);


#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
