/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_RENDER_H__
#define __KIBA_RENDER_H__

#include "../config.h"
#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#ifdef HAVE_SVG
  #include <librsvg/rsvg.h>
  #include <librsvg/rsvg-cairo.h>
#endif

typedef struct _KibaSurface KibaSurface;

#include "kiba.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	KIBA_OBJECTFG_SURFACE = 0,  /* keep position */
	KIBA_OBJECTBG_SURFACE,      /* keep position */
	KIBA_DOCK_SURFACE,
	KIBA_TITLE_SURFACE,
	KIBA_SEPARATOR_SURFACE,
	KIBA_TRIANGLE_SURFACE,
	KIBA_BUBBLE_SURFACE,
	KIBA_PROGRESS_SURFACE,
	KIBA_ARROW_SURFACE
} KibaSurfaceType;

struct _KibaSurface {
	KibaSurfaceType   type;
	cairo_surface_t  *surface;
	gboolean          needs_update;
	double            width;
	double            height;
};

void kiba_surface_init (KibaSurface     **surface,
			KibaSurfaceType   type,
			double            width,
			double            height);

void kiba_surface_fini (KibaSurface **surface);

/**
 * Draw Shape on Context.
 */
void kiba_paint_shape (KibaLayoutData *d,
                       cairo_t        *draw_context,
                       const char     *topleft_corner,
                       const char     *topright_corner,
                       const char     *bottomleft_corner,
                       const char    *bottomright_corner,
                       gboolean       dock_shape,
                       gboolean       draw_all_border,
                       double         x0,
                       double         y0,
                       double         x1,
                       double         y1,
                       double         radius,
                       double         border_width);

/**
 * Get Gradient Pattern.
 */
cairo_pattern_t* kiba_get_gradient_pattern (KibaLayoutData *d,
                                            const char     *rgba_1,
                                            const char     *rgba_2,
                                            const char     *rgba_3,
                                            double          x0,
                                            double          x1,
                                            double          y0,
                                            double          y1);

/**
 * Get Pattern for PNG File.
 *
 * @param cr The Cairo Context.
 * @param png_file The PNG File.
 * @param rotate.
 * @return The cairo_pattern_t created.
 */
cairo_pattern_t* kiba_get_png_pattern (cairo_t    *cr,
                                       const char *png_file,
                                       gboolean    rotate);

/**
 * Get Linear/Radial Pattern.
 */
cairo_pattern_t* kiba_get_bg_pattern (KibaLayoutData *d,
				      GtkAllocation   allocation,
				      gboolean        linear,
				      const char     *rgba_1,
				      const char     *rgba_2,
				      const char     *rgba_3,
				      double          affection,
				      double          step_size,
				      double          radius_1,
				      double          radius_2);

/**
 * Update PangoLayout.
 *
 * @param geometry The geometry of the dest surface.
 * @param text The new text.
 * @param description The new Description.
 * @param cr The Drawing Context.
 * @param layout The PangoLayout.
 */
void kiba_update_pango_layout (GdkRectangle  geometry,
			       const char   *text,
			       const char   *description,
			       cairo_t      *cr,
			       PangoLayout **layout);

/**
 * Load Icon File at Path in the Pixbuf.
 *
 * @param icon The PNG Icon Name, or the Absolute Path.
 * @param size The Pixbuf size.
 * @param pixbuf The Pixbuf.
 * @return @c TRUE on success, otherwise
 *         @c FALSE.
 */
gboolean kiba_load_icon (const char *icon,
			 int         size,
			 GdkPixbuf **pixbuf);

#ifdef HAVE_SVG
/**
 * Load SVG File at Path in the RsvgHandle.
 *
 * @param icon The SVG Icon Name, or the Absolute Path.
 * @param size The SVG size.
 * @param pixbuf The RsvgHandle.
 * @return @c TRUE on success, otherwise
 *         @c FALSE.
 */
gboolean kiba_load_svg (const char  *icon,
                        double      *size,
                        RsvgHandle **handle);
#endif

/**
 * Switch pointer of Surfaces.
 *
 * @param flip_surface A cairo_surface_t.
 * @param flop_surface A cairo_surface_t.
 */
void kiba_switch_surfaces (cairo_surface_t **flip_surface,
                           cairo_surface_t **flop_surface);

void kiba_rotate_context (cairo_t      *cr,
			  double        width,
			  double        height,
			  GdkPoint     anchor);

/**
 * Get Red, Green, Blue double Values from String.
 *
 * @param html_color The RGB-String.
 * @param r The Red Value.
 * @param g The Green Value.
 * @param b The Blue Value.
 */
void kiba_get_rgb_from_string (const char *html_color,
                               double     *r,
                               double     *g,
                               double     *b);

/**
 * Get Red, Green, Blue, Alpha double Values from String.
 *
 * @param html_color The RGBA-String.
 * @param r The Red Value.
 * @param g The Green Value.
 * @param b The Blue Value.
 * @param alpha The Alpha Value.
 */
void kiba_get_rgba_from_string (const char *html_color,
                                double     *r,
                                double     *g,
                                double     *b,
                                double     *alpha);

/**
 * Set Cairo Context RGBA from string.
 *
 * @param cr The Cairo Context.
 * @param alpha_factor Alpha Factor.
 * @param rgba The RGBA String (#FFFFFFFF for 100% opacity and white color).
 */
void cairo_set_source_rgba_from_string (cairo_t    *cr,
					double      alpha_factor,
					const char *rgba);

/**
 * Rerender Kiba.
 *
 * @param kiba_widget The kiba Widget.
 */
void kiba_render (GtkWidget *kiba_widget);

/**
 * Clear the Kiba Window.
 *
 * @param kiba_widget The kiba Widget.
 */
void kiba_clear_window (GtkWidget *kiba_widget);

/**
 * Update Kiba Surface Bufers.
 *
 * @param kiba_widget The kiba Widget.
 */
void kiba_update_surface_buffers (GtkWidget *kiba_widget);

/**
 * Update Kiba Surface Bufers.
 *
 * @param kiba_widget The kiba Widget.
 * @return The cairo_t created.
 */
cairo_t* kiba_cairo_create (GtkWidget *kiba_widget);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif


#endif
